import * as React from 'react';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { History } from 'history';
import OutsideClickHandler from 'react-outside-click-handler';
import { last, slice } from 'ramda';

import { isNilOrEmpty } from '@romanjaros/treejs-utils';

import Modals from '../../Modal/components/Modals';
import Toasters from '../../Toaster/components/Toasters';

import { IMenuItem } from '../types';
import {
	StyledLeftMenu,
	StyledLeftMenuLogo,
	StyledPageContent,
	StyledRightSide,
	StyledTopMenu,
	StyledTopMenuItem,
	StyledTopMenuItemHamburger,
	StyledTopMenuItemLogo,
	StyledTopMenuSubMenuContainer,
	StyledTopMenuSubMenuItem,
	StyledUserContainer,
} from '../style';
import { getComputedStyleInt } from '../utils';

import MenuItem from './MenuItem';
import MenuDropdown from './MenuDropdown';


interface IProps {
	enableLeftMenu: boolean;
	enableUserMenu: boolean;
	history: History;
	leftMenuItems?: IMenuItem[];
	logo: React.ReactElement;
	topMenuItems?: IMenuItem[];
	userComponent?: React.ReactElement;
	userMenuItems?: IMenuItem[];
}

interface IState {
	isLeftMenuOpen: boolean;
	isSubMenu: boolean;
	isSubMenuOpen: boolean;
	topMenuItems: IMenuItem[];
	topMenuMissingItems: IMenuItem[];
}

class PageLayout extends React.PureComponent<IProps, IState> {
	public static defaultProps = {
		enableLeftMenu: true,
	};

	public readonly state = {
		isSubMenu: null as any,
		isSubMenuOpen: false,
		topMenuItems: this.props.topMenuItems,
		topMenuMissingItems: [] as IMenuItem[],
		isLeftMenuOpen: false,
	};

	private topMenuItemsWidth: number[] = [];

	public componentDidMount() {
		window.addEventListener("resize", this.recalculateMenu);
		this.topMenuItemsWidth = this.getTopLeftMenuItemsWidth();
		this.recalculateMenu();
	}

	public componentDidUpdate(prevProps: IProps) {
		if (prevProps.topMenuItems !== this.props.topMenuItems || isNilOrEmpty(this.state.isSubMenu)) {
			this.topMenuItemsWidth = this.getTopLeftMenuItemsWidth();
			this.recalculateMenu();
		}
	}

	public componentWillUnmount() {
		window.removeEventListener("resize", this.recalculateMenu);
	}

	private recalculateMenu = () => {
		const menuWidth = this.topMenuRef.current.offsetWidth - 90;
		let topMenuItemWidth = last(this.topMenuItemsWidth);

		// TODO when is not user section in menu
		for (let i = 0; i < this.topMenuItemsWidth.length - 1; i++) {
			const itemWidth = this.topMenuItemsWidth[i];
			topMenuItemWidth += itemWidth;
			if (topMenuItemWidth >= menuWidth) {
				this.setState({
					topMenuItems: slice(0, i - 1, this.props.topMenuItems),
					topMenuMissingItems: slice(i - 1, Infinity, this.props.topMenuItems),
					isSubMenu: true,
				});
				return;
			}
		}

		this.setState({
			topMenuItems: this.props.topMenuItems,
			isSubMenu: false,
			isSubMenuOpen: false,
			topMenuMissingItems: [],
		});
	};

	private getTopLeftMenuItemsWidth = () => {
		const children = this.topMenuRef.current.children;
		const sizes = [];
		for (let i = 0; i < children.length; i++) {
			const child = children[i];
			const margin = getComputedStyleInt(child, 'marginLeft') + getComputedStyleInt(child, 'marginRight');
			// TODO when is not user section in menu
			sizes.push(getComputedStyleInt(child, 'width') + (i === children.length - 1 ? 0 : margin));
		}

		return sizes;
	};

	private openLeftMenu = () => {
		if (!this.state.isLeftMenuOpen) {
			this.setState({
				isLeftMenuOpen: true,
			});
		}
	};

	private closeLeftMenu = () => {
		if (this.state.isLeftMenuOpen) {
			this.setState({
				isLeftMenuOpen: false,
			});
		}
	};

	private handleToggleSubMenu = () => {
		this.setState({
			isSubMenuOpen: !this.state.isSubMenuOpen,
		});
	};

	private handleCloseSubMenu = () => {
		this.setState({
			isSubMenuOpen: false,
		});
	};

	private openPage = (href: string) => {
		this.props.history.push(href);
		this.setState({
			isLeftMenuOpen: false,
		});
	};

	private handleOpenPage = (href: string) => () => {
		this.openPage(href);
	};

	private topMenuRef: React.RefObject<HTMLDivElement> = React.createRef();

	public render(): React.ReactElement {
		const {
			leftMenuItems,
			children,
			logo,
			enableUserMenu,
			userComponent,
			userMenuItems,
			enableLeftMenu,
		} = this.props;

		const {
			isLeftMenuOpen,
			topMenuItems,
			topMenuMissingItems,
			isSubMenu,
			isSubMenuOpen,
		} = this.state;

		return (
			<React.Fragment>
				<StyledLeftMenu open={isLeftMenuOpen} enableLeftMenu={enableLeftMenu}>
					<OutsideClickHandler onOutsideClick={this.closeLeftMenu}>
						{
							enableLeftMenu && (
								<StyledLeftMenuLogo>
									{logo}
								</StyledLeftMenuLogo>
							)
						}
						{
							leftMenuItems && leftMenuItems.map(
								(menuItem, i) => (
									<MenuItem
										key={i}
										isChild={false}
										onClick={this.openPage}
										{...menuItem}
									/>
								)
							)
						}
					</OutsideClickHandler>
				</StyledLeftMenu>
				<StyledRightSide open={isLeftMenuOpen} enableLeftMenu={enableLeftMenu}>
					<OutsideClickHandler onOutsideClick={this.handleCloseSubMenu}>
						<StyledTopMenu ref={this.topMenuRef}>
							{
								!enableLeftMenu && (
									<StyledTopMenuItemLogo>
										{logo}
									</StyledTopMenuItemLogo>
								)
							}
							{
								!isLeftMenuOpen && enableLeftMenu && (
									<StyledTopMenuItemHamburger onClick={this.openLeftMenu}>
										<FontAwesomeIcon icon="bars" />
									</StyledTopMenuItemHamburger>
								)
							}
							{
								topMenuItems && topMenuItems.map(
									({ label, href }, i) => (
										<StyledTopMenuItem key={i} onClick={this.handleOpenPage(href)}>
											{label}
										</StyledTopMenuItem>
									)
								)
							}
							{
								isSubMenu && !isNilOrEmpty(topMenuMissingItems) && (
									<StyledTopMenuItem onClick={this.handleToggleSubMenu}>
										<FontAwesomeIcon icon={isSubMenuOpen ? 'chevron-up' : 'chevron-down'} />
									</StyledTopMenuItem>
								)
							}
							{
								enableUserMenu && !isNilOrEmpty(userComponent) && (
									<StyledUserContainer>
										<MenuDropdown menuItems={userMenuItems}>
											{userComponent}
										</MenuDropdown>
									</StyledUserContainer>
								)
							}
						</StyledTopMenu>
						{
							isSubMenuOpen && !isNilOrEmpty(topMenuMissingItems) && (
								<StyledTopMenuSubMenuContainer>
									{
										topMenuMissingItems.map(
											({ label, href }, i) => (
												<StyledTopMenuSubMenuItem key={i} onClick={this.handleOpenPage(href)}>
													{label}
												</StyledTopMenuSubMenuItem>
											)
										)
									}
								</StyledTopMenuSubMenuContainer>
							)
						}
					</OutsideClickHandler>
					<StyledPageContent open={isLeftMenuOpen} enableLeftMenu={enableLeftMenu}>
						{children}
					</StyledPageContent>
				</StyledRightSide>

				<Modals />
				<Toasters />
			</React.Fragment>
		);
	}
}

export default PageLayout;
